require "thread"
require "socket"
    
ChessServer     = "127.0.0.1"  # Replace this IP address!
ChessServerPort = 12000

# Exit if user presses Enter at server end
waiter = Thread.new do
  puts "Press Enter to exit the server."
  gets
  exit
end
    
$mutex = Mutex.new
$list = {}

Player = Struct.new("Player", :opponent, :address,
                    :port, :ipname, :session, 
                    :short, :long, :id)

def match?(p1, p2)
  $list[p1] and $list[p2] and
    $list[p1][0] == p2 and $list[p2][0] == p1
end

def handle_client(sess, msg, addr, port, ipname)
  $mutex.synchronize do
    cmd, p1short, p2long = msg.split
    if cmd != "login"
      puts "Protocol error: client msg was #{msg}"
      return
    end

    p1long = p1short.dup + ":#{addr}"
    p2short, host2 = p2long.split(":")
    host2 = ipname if host2 == nil
    p2long = p2short + ":" + IPSocket.getaddress(host2)

    p1 = Struct::Player.new(p2long, addr, port, ipname, 
                            sess, p1short, p1long)
    p2 = Struct::Player.new

    # Note: We get user:hostname on the command line,
    # but we store it in the form user:address

    $list[p1long] = p1

    if match?(p1long, p2long)
      # Note these names are "backwards" now: player 2 
      # logged in first, if we got here.
      p1, p2 = $list[p1long], $list[p2long]
      # Player ID = name:ipname:color
      # Color: 0=white, 1=black
      p1.id = "#{p1short}:#{p1.ipname}:1"
      p2.id = "#{p2short}:#{p2.ipname}:0"
      sess1, sess2 = [p1.session, p2.session]
      sess1.puts "#{p2.id}"
      sess1.close
      sess2.puts "#{p1.id}"
      sess2.close
    end 
  end
end

text = nil

$server = TCPServer.new(ChessServer, ChessServerPort)
while session = $server.accept do
  Thread.new(session) do |sess|
    text = sess.gets
    puts "Received: #{text}"  # So we know server gets it
    domain, port, ipname, ipaddr = sess.peeraddr
    handle_client sess, text, ipaddr, port, ipname
    sleep 1
  end
end

waiter.join    # Exit if user presses Enter (this is only
               # for convenience in running this example)
